package com.tofly.dict.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.dict.entity.SysDict;
import com.tofly.dict.entity.dto.BaseDto;
import com.tofly.dict.entity.dto.DictDto;
import com.tofly.dict.service.SysDictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 系统字典(SysDict)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 17:41:22
 */
@Api(tags = "系统字典")
@RestController
@RequestMapping("sysDict")
public class SysDictController {
    /**
     * 服务对象
     */
    @Resource
    private SysDictService sysDictService;

    /**
     * 分页查询所有数据
     *
     * @param page    分页对象
     * @param sysDict 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultRespone pageList(Page<SysDict> page, SysDict sysDict) {
        return ResultRespone.success(sysDictService.page(page, new QueryWrapper<>(sysDict).lambda().orderByAsc(SysDict::getSort)));
    }

    /**
     * 查询所有数据
     *
     * @param sysDict 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "sysDict", value = "查询字段模型")
    })
    @GetMapping("/allList")
    public ResultRespone list(SysDict sysDict) {
        //根据code查询子集
        if (sysDict != null && StringUtils.hasText(sysDict.getCode())) {
            List<SysDict> list = sysDictService.list(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getCode, sysDict.getCode()).eq(SysDict::getPid, 0));
            if (!CollectionUtils.isEmpty(list)) {
                sysDict.setCode(null);
                return ResultRespone.success(sysDictService.list(new QueryWrapper<SysDict>(sysDict).lambda().eq(SysDict::getPid, list.get(0).getId()).orderByAsc(SysDict::getSort)));
            }
        }
        return ResultRespone.success(sysDictService.list(new QueryWrapper<>(sysDict).lambda().orderByAsc(SysDict::getSort)));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone selectOne(@PathVariable Serializable id) {
        return ResultRespone.success(sysDictService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param dictDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody DictDto dictDto) {
        sysDictService.insert(dictDto);
        return ResultRespone.success("新增成功");
    }

    /**
     * 修改数据
     *
     * @param dictDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone update(@Validated(value = BaseDto.Update.class) @RequestBody DictDto dictDto) {
        sysDictService.edit(dictDto);
        return ResultRespone.success("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        for (Long id : idList) {
            if (sysDictService.count(Wrappers.<SysDict>lambdaQuery().eq(SysDict::getPid, id)) != 0) {
                throw new RuntimeException("所选字典下面包含子集，请先删除子集字典");
            }
        }
        return ResultRespone.success(sysDictService.removeByIds(idList));
    }

    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
//        sysDictService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
//        return decide(sysDictService.importData(file));
        return ResultRespone.success("导入成功");
    }


    /**
     * 数据导出
     *
     * @param page    分页
     * @param sysDict 查询模型
     * @param isAll   是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<SysDict> page, SysDict sysDict, Boolean isAll) {
//        sysDictService.export(page,sysDict,isAll);
    }
}

